/**
 * 使用方法
 * 引用
 * import { IDCardData } from "@/common/utils/id_card_area";
 * 调用
 * IDCardData("身份证号",true)
 * 参数： 身份证号[必填]  是否验证[非必填]
 */

/**
 * 行政区划数据
 * 根据身份证号取得发证地，身份证号前六位映射到地区的结果例如“110101--北京市东城区”“411202--河南省三门峡市湖滨区”
 */
import area from "./area";

/**
 * 身份信息获取
 * @param {*} idCard 身份证号
 */
export const getAreaByIdCard = (idCard) => {
  //籍贯
  if (!idCard || idCard.length < 6) return "";
  let provinceNo = idCard.substring(0, 2) + "0000";
  let cityNo = idCard.substring(0, 4) + "00";
  let countyNo = idCard.substring(0, 6);

  let provinceName = area.province[provinceNo];
  let cityName = area.city[cityNo];
  let countyName = area.county[countyNo];

  let Hometown = {
    provinceName,
    cityName,
    countyName,
    address: `${provinceName}-${cityName}-${countyName}`,
  };

  //出生日期
  let Birthday = {
    date: `${idCard.substring(6, 10)}-${idCard.substring(
      10,
      12
    )}-${idCard.substring(12, 14)}`,
    year: idCard.substring(6, 10),
    month: idCard.substring(10, 12),
    day: idCard.substring(12, 14),
  };

  //性别
  let sex = parseInt(idCard.substr(16, 1)) % 2 == 1 ? "男" : "女";

  //年龄
  let age = new Date().getFullYear() - idCard.substring(6, 10) - 1;

  return {
    Hometown,
    Birthday,
    sex,
    age,
    idCard,
  };
};

/**
 * 严格验证身份证号
 * @param {*} idcard 身份证号
 */
export const checkIdcard = (idcard) => {
  var Errors = new Array(
    "验证通过!",
    "身份证号码位数不对!",
    "身份证号码出生日期超出范围或含有非法字符!",
    "身份证号码校验错误!",
    "身份证地区非法!"
  );
  var area = {
    11: "北京",
    12: "天津",
    13: "河北",
    14: "山西",
    15: "内蒙古",
    21: "辽宁",
    22: "吉林",
    23: "黑龙江",
    31: "上海",
    32: "江苏",
    33: "浙江",
    34: "安徽",
    35: "福建",
    36: "江西",
    37: "山东",
    41: "河南",
    42: "湖北",
    43: "湖南",
    44: "广东",
    45: "广西",
    46: "海南",
    50: "重庆",
    51: "四川",
    52: "贵州",
    53: "云南",
    54: "西藏",
    61: "陕西",
    62: "甘肃",
    63: "青海",
    64: "宁夏",
    65: "新疆",
    71: "台湾",
    81: "香港",
    82: "澳门",
    91: "国外",
  };
  var idcard, Y, JYM, ereg;
  var S, M;
  var idcard_array = new Array();
  idcard_array = idcard.split("");
  //地区检验
  if (area[parseInt(idcard.substr(0, 2))] == null) {
    return {
      msg: Errors[4],
      status: false,
    };
  }
  //身份号码位数及格式检验
  switch (idcard.length) {
    case 15:
      if (
        (parseInt(idcard.substr(6, 2)) + 1900) % 4 == 0 ||
        ((parseInt(idcard.substr(6, 2)) + 1900) % 100 == 0 &&
          (parseInt(idcard.substr(6, 2)) + 1900) % 4 == 0)
      ) {
        ereg = /^[1-9][0-9]{5}[0-9]{2}((01|03|05|07|08|10|12)(0[1-9]|[1-2][0-9]|3[0-1])|(04|06|09|11)(0[1-9]|[1-2][0-9]|30)|02(0[1-9]|[1-2][0-9]))[0-9]{3}$/; //测试出生日期的合法性
      } else {
        ereg = /^[1-9][0-9]{5}[0-9]{2}((01|03|05|07|08|10|12)(0[1-9]|[1-2][0-9]|3[0-1])|(04|06|09|11)(0[1-9]|[1-2][0-9]|30)|02(0[1-9]|1[0-9]|2[0-8]))[0-9]{3}$/; //测试出生日期的合法性
      }
      if (ereg.test(idcard)) {
        return {
          msg: Errors[0],
          status: true,
        };
      } else {
        return {
          msg: Errors[2],
          status: false,
        };
      }
      break;
    case 18:
      //18位身份号码检测
      //出生日期的合法性检查
      //闰年月日:((01|03|05|07|08|10|12)(0[1-9]|[1-2][0-9]|3[0-1])|(04|06|09|11)(0[1-9]|[1-2][0-9]|30)|02(0[1-9]|[1-2][0-9]))
      //平年月日:((01|03|05|07|08|10|12)(0[1-9]|[1-2][0-9]|3[0-1])|(04|06|09|11)(0[1-9]|[1-2][0-9]|30)|02(0[1-9]|1[0-9]|2[0-8]))
      if (
        parseInt(idcard.substr(6, 4)) % 4 == 0 ||
        (parseInt(idcard.substr(6, 4)) % 100 == 0 &&
          parseInt(idcard.substr(6, 4)) % 4 == 0)
      ) {
        ereg = /^[1-9][0-9]{5}19[0-9]{2}((01|03|05|07|08|10|12)(0[1-9]|[1-2][0-9]|3[0-1])|(04|06|09|11)(0[1-9]|[1-2][0-9]|30)|02(0[1-9]|[1-2][0-9]))[0-9]{3}[0-9Xx]$/; //闰年出生日期的合法性正则表达式
      } else {
        ereg = /^[1-9][0-9]{5}19[0-9]{2}((01|03|05|07|08|10|12)(0[1-9]|[1-2][0-9]|3[0-1])|(04|06|09|11)(0[1-9]|[1-2][0-9]|30)|02(0[1-9]|1[0-9]|2[0-8]))[0-9]{3}[0-9Xx]$/; //平年出生日期的合法性正则表达式
      }

      if (ereg.test(idcard)) {
        //测试出生日期的合法性
        //计算校验位
        S =
          (parseInt(idcard_array[0]) + parseInt(idcard_array[10])) * 7 +
          (parseInt(idcard_array[1]) + parseInt(idcard_array[11])) * 9 +
          (parseInt(idcard_array[2]) + parseInt(idcard_array[12])) * 10 +
          (parseInt(idcard_array[3]) + parseInt(idcard_array[13])) * 5 +
          (parseInt(idcard_array[4]) + parseInt(idcard_array[14])) * 8 +
          (parseInt(idcard_array[5]) + parseInt(idcard_array[15])) * 4 +
          (parseInt(idcard_array[6]) + parseInt(idcard_array[16])) * 2 +
          parseInt(idcard_array[7]) * 1 +
          parseInt(idcard_array[8]) * 6 +
          parseInt(idcard_array[9]) * 3;
        Y = S % 11;
        M = "F";
        JYM = "10X98765432";
        M = JYM.substr(Y, 1); //判断校验位
        if (M == idcard_array[17]) {
          //检测ID的校验位
          return {
            msg: Errors[0],
            status: true,
          };
        } else {
          return {
            msg: Errors[3],
            status: false,
          };
        }
      } else {
        return {
          msg: Errors[2],
          status: false,
        };
      }
      break;
    default:
      return {
        msg: Errors[1],
        status: false,
      };
      break;
  }
};

/**
 * 根据身份证号取得发证地，身份证号前六位映射到地区的结果例如“110101--北京市东城区”“411202--河南省三门峡市湖滨区”
 * @param {String} idCard 身份证号
 * @param {Boolean} verification 是否验证身份证号
 */
export const IDCardData = (idCard, verification) => {
  if (verification) {
    if (checkIdcard(idCard).status) {
      return {
        ...checkIdcard(idCard),
        data: getAreaByIdCard(idCard),
      };
    } else {
      return checkIdcard(idCard);
    }
  } else {
    return getAreaByIdCard(idCard);
  }
};
